#include "../../BarinkEngine/src/BarinkEngine.h"
#include "../../BarinkEngine/src/Scene/Components.h"
#include "../../BarinkEngine/src/Scene/Scene.h"
#include "../../BarinkEngine/src/Scene/Entity.h"
#include "../../BarinkEngine/src/AssetManager/ModelImporter.h"
#include <imgui.h>
#include "GUI.h"
#include "Util.h"
#include <entt/entt.hpp>

/*
* Define globals
*/
Scene scene;

BarinkEngine::Renderable* renderable;
BarinkEngine::SceneObject* object;
Entity cube;
/*
* Runs once at startup 
* - USe to initialize the game/sandbox/demo
*/
void Start() {
   
    cube  = scene.AddEntity((std::string&)"cube");
    auto& render3DComponent = cube.AddComponent<BarinkEngine::Render3DComponent>();
    auto importer = BarinkEngine::ModelImporter();

    object = importer.Import("build/Debug/Models/Cube.obj");
    renderable = object->renderable;

    render3DComponent.mesh = *renderable->mesh;
    cube.GetComponent<BarinkEngine::TransformComponent>()
        .transform = glm::rotate(glm::mat4(1.0f), 32.0f, glm::vec3(0.5f,1.0f,0.0f));

    // Create a second cube
   
    auto cube2 = scene.AddEntity((std::string&)"Cube2");
    auto& cube2Render = cube2.AddComponent<BarinkEngine::Render3DComponent>();
    cube2Render.mesh = *renderable->mesh;
    cube2Render.color = glm::vec3(0.0f, 1.0f, 0.0f);
    auto& cube2Trans = cube2.GetComponent<BarinkEngine::TransformComponent>();
    cube2Trans.transform = glm::translate( glm::mat4(1.0f), glm::vec3(1.0f,0.0f, 5.0f));

   
    renderer.Prepare(scene);



}





/*
* Runs every frame
* - Use to draw Immediate mode graphics (Not meant for HUD's )
*/
void ImmediateGraphicsDraw() 
{
    // Show internal BarinkEngine stats
    ShowStats();

    ImGui::Begin("Render edit");
        
    auto& a = cube.GetComponent<BarinkEngine::Render3DComponent>();
    
    ImGui::DragFloat3("Color", &a.color[0], 0.01f, 0.0f, 1.0f);

    ImGui::End();




}

/*
* Runs every frame
* - Meant for game logic ( non-physics related) 
*/
void Update()
{
    renderer.Render(scene);
   
}

/*
* Runs at the end of the program
* - Meant for cleanup
*/
void Stop()
{
}